@@ -1,6 +1,7 @@
 from asyncio import iscoroutinefunction
 from collections.abc import Callable, Coroutine, Iterable, MutableMapping
 from contextvars import ContextVar, Token
+from threading import Lock
 from types import TracebackType
 from typing import Any, Self, cast, final, overload
 
@@ -24,7 +25,10 @@ class ScopeState:
     This class is immutable after initialization.
     """
 
-    __slots__ = ("_state",)
+    __slots__ = (
+        "_lock",
+        "_state",
+    )
 
     def __init__(
         self,
@@ -36,6 +40,12 @@ class ScopeState:
             "_state",
             {type(element): element for element in state},
         )
+        self._lock: Lock
+        object.__setattr__(
+            self,
+            "_lock",
+            Lock(),
+        )
 
     def __setattr__(
         self,
@@ -56,6 +66,51 @@ class ScopeState:
             f" attribute - '{name}' cannot be deleted"
         )
 
+    def check_state[StateType: State](
+        self,
+        state: type[StateType],
+        /,
+        *,
+        instantiate_defaults: bool = False,
+    ) -> bool:
+        """
+        Check state object availability by its type.
+
+        If the state type is not found, attempts to instantiate a new instance of\
+         the type if possible.
+
+        Parameters
+        ----------
+        state: type[StateType]
+            The type of state to check
+
+        instantiate_defaults: bool = False
+            Control if default value should be instantiated during check.
+
+        Returns
+        -------
+        bool
+            True if state is available, otherwise False.
+        """
+        if state in self._state:
+            return True
+
+        elif instantiate_defaults:
+            with self._lock:
+                if state in self._state:
+                    return True
+
+                try:
+                    initialized: StateType = state()
+                    self._state[state] = initialized
+                    return True
+
+                except BaseException:
+                    return False  # unavailable, we don't care the exception
+
+        else:
+            return False
+
     def state[StateType: State](
         self,
         state: type[StateType],
@@ -93,17 +148,20 @@ class ScopeState:
             return default
 
         else:
-            try:
-                initialized: StateType = state()
-                # we would need a locking here in multithreaded environment
-                self._state[state] = initialized
-                return initialized
+            with self._lock:
+                if state in self._state:
+                    return cast(StateType, self._state[state])
 
-            except Exception as exc:
-                raise MissingState(
-                    f"{state.__qualname__} is not defined in current scope"
-                    " and failed to provide a default value"
-                ) from exc
+                try:
+                    initialized: StateType = state()
+                    self._state[state] = initialized
+                    return initialized
+
+                except Exception as exc:
+                    raise MissingState(
+                        f"{state.__qualname__} is not defined in current scope"
+                        " and failed to provide a default value"
+                    ) from exc
 
     def updated(
         self,
@@ -149,6 +207,40 @@ class StateContext:
 
     _context = ContextVar[ScopeState]("StateContext")
 
+    @classmethod
+    def check_state[StateType: State](
+        cls,
+        state: type[StateType],
+        /,
+        instantiate_defaults: bool = False,
+    ) -> bool:
+        """
+        Check if state object is available in the current context.
+
+        Verifies if state object of the specified type is available the current context.
+
+        Parameters
+        ----------
+        state: type[StateType]
+            The type of state to check
+
+        instantiate_defaults: bool = False
+            Control if default value should be instantiated during check.
+
+        Returns
+        -------
+        bool
+            True if state is available, otherwise False.
+        """
+        try:
+            return cls._context.get().check_state(
+                state,
+                instantiate_defaults=instantiate_defaults,
+            )
+
+        except LookupError:
+            return False  # no context no state
+
     @classmethod
     def state[StateType: State](
         cls,
@@ -182,7 +274,10 @@ class StateContext:
             If state not found and default not provided or instantiation fails
         """
         try:
-            return cls._context.get().state(state, default=default)
+            return cls._context.get().state(
+                state,
+                default=default,
+            )
 
         except LookupError as exc:
             raise MissingContext("StateContext requested but not defined!") from exc
